/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define GPIOA_IRQHandler gpioa_isr
#define GPIOB_IRQHandler gpiob_isr
#define GPIOC_IRQHandler gpioc_isr
#define GPIOD_IRQHandler gpiod_isr
#define GPIOE_IRQHandler gpioe_isr
#define UART0_IRQHandler uart0_isr
#define UART1_IRQHandler uart1_isr
#define SSI0_IRQHandler ssi0_isr
#define I2C0_IRQHandler i2c0_isr
#define PWM0_FAULT_IRQHandler pwm0_fault_isr
#define PWM0_0_IRQHandler pwm0_0_isr
#define PWM0_1_IRQHandler pwm0_1_isr
#define PWM0_2_IRQHandler pwm0_2_isr
#define QEI0_IRQHandler qei0_isr
#define ADC0SS0_IRQHandler adc0ss0_isr
#define ADC0SS1_IRQHandler adc0ss1_isr
#define ADC0SS2_IRQHandler adc0ss2_isr
#define ADC0SS3_IRQHandler adc0ss3_isr
#define WATCHDOG_IRQHandler watchdog_isr
#define TIMER0A_IRQHandler timer0a_isr
#define TIMER0B_IRQHandler timer0b_isr
#define TIMER1A_IRQHandler timer1a_isr
#define TIMER1B_IRQHandler timer1b_isr
#define TIMER2A_IRQHandler timer2a_isr
#define TIMER2B_IRQHandler timer2b_isr
#define COMP0_IRQHandler comp0_isr
#define COMP1_IRQHandler comp1_isr
#define COMP2_IRQHandler comp2_isr
#define SYSCTL_IRQHandler sysctl_isr
#define FLASH_IRQHandler flash_isr
#define GPIOF_IRQHandler gpiof_isr
#define GPIOG_IRQHandler gpiog_isr
#define GPIOH_IRQHandler gpioh_isr
#define UART2_IRQHandler uart2_isr
#define SSI1_IRQHandler ssi1_isr
#define TIMER3A_IRQHandler timer3a_isr
#define TIMER3B_IRQHandler timer3b_isr
#define I2C1_IRQHandler i2c1_isr
#define QEI1_IRQHandler qei1_isr
#define CAN0_IRQHandler can0_isr
#define CAN1_IRQHandler can1_isr
#define CAN2_IRQHandler can2_isr
#define ETH_IRQHandler eth_isr
#define HIBERNATE_IRQHandler hibernate_isr
#define USB0_IRQHandler usb0_isr
#define PWM0_3_IRQHandler pwm0_3_isr
#define UDMA_IRQHandler udma_isr
#define UDMAERR_IRQHandler udmaerr_isr
#define ADC1SS0_IRQHandler adc1ss0_isr
#define ADC1SS1_IRQHandler adc1ss1_isr
#define ADC1SS2_IRQHandler adc1ss2_isr
#define ADC1SS3_IRQHandler adc1ss3_isr
#define I2S0_IRQHandler i2s0_isr
#define EPI0_IRQHandler epi0_isr
#define GPIOJ_IRQHandler gpioj_isr
#define GPIOK_IRQHandler gpiok_isr
#define GPIOL_IRQHandler gpiol_isr
#define SSI2_IRQHandler ssi2_isr
#define SSI3_IRQHandler ssi3_isr
#define UART3_IRQHandler uart3_isr
#define UART4_IRQHandler uart4_isr
#define UART5_IRQHandler uart5_isr
#define UART6_IRQHandler uart6_isr
#define UART7_IRQHandler uart7_isr
#define I2C2_IRQHandler i2c2_isr
#define I2C3_IRQHandler i2c3_isr
#define TIMER4A_IRQHandler timer4a_isr
#define TIMER4B_IRQHandler timer4b_isr
#define TIMER5A_IRQHandler timer5a_isr
#define TIMER5B_IRQHandler timer5b_isr
#define WTIMER0A_IRQHandler wtimer0a_isr
#define WTIMER0B_IRQHandler wtimer0b_isr
#define WTIMER1A_IRQHandler wtimer1a_isr
#define WTIMER1B_IRQHandler wtimer1b_isr
#define WTIMER2A_IRQHandler wtimer2a_isr
#define WTIMER2B_IRQHandler wtimer2b_isr
#define WTIMER3A_IRQHandler wtimer3a_isr
#define WTIMER3B_IRQHandler wtimer3b_isr
#define WTIMER4A_IRQHandler wtimer4a_isr
#define WTIMER4B_IRQHandler wtimer4b_isr
#define WTIMER5A_IRQHandler wtimer5a_isr
#define WTIMER5B_IRQHandler wtimer5b_isr
#define SYSEXC_IRQHandler sysexc_isr
#define PECI0_IRQHandler peci0_isr
#define LPC0_IRQHandler lpc0_isr
#define I2C4_IRQHandler i2c4_isr
#define I2C5_IRQHandler i2c5_isr
#define GPIOM_IRQHandler gpiom_isr
#define GPION_IRQHandler gpion_isr
#define FAN0_IRQHandler fan0_isr
#define GPIOP0_IRQHandler gpiop0_isr
#define GPIOP1_IRQHandler gpiop1_isr
#define GPIOP2_IRQHandler gpiop2_isr
#define GPIOP3_IRQHandler gpiop3_isr
#define GPIOP4_IRQHandler gpiop4_isr
#define GPIOP5_IRQHandler gpiop5_isr
#define GPIOP6_IRQHandler gpiop6_isr
#define GPIOP7_IRQHandler gpiop7_isr
#define GPIOQ0_IRQHandler gpioq0_isr
#define GPIOQ1_IRQHandler gpioq1_isr
#define GPIOQ2_IRQHandler gpioq2_isr
#define GPIOQ3_IRQHandler gpioq3_isr
#define GPIOQ4_IRQHandler gpioq4_isr
#define GPIOQ5_IRQHandler gpioq5_isr
#define GPIOQ6_IRQHandler gpioq6_isr
#define GPIOQ7_IRQHandler gpioq7_isr
#define PWM1_0_IRQHandler pwm1_0_isr
#define PWM1_1_IRQHandler pwm1_1_isr
#define PWM1_2_IRQHandler pwm1_2_isr
#define PWM1_3_IRQHandler pwm1_3_isr
#define PWM1_FAULT_IRQHandler pwm1_fault_isr
