/** @page libopencm3 LPC17xx

@version 1.0.0

@date 14 September 2012

API documentation for NXP Semiconductors LPC17xx Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC17xx LPC17xx
Libraries for NXP Semiconductors LPC17xx series.

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC17xx_defines LPC17xx Defines

@brief Defined Constants and Types for the LPC17xx series

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

