/** @page libopencm3 NRF51

@version 1.0.0

@date April 2020

API documentation for Nordic Semiconductor NRF51 Cortex M0 series

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup NRF51xx NRF51xx
Libraries for Nordic Semiconductor NRF51xx series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup NRF51xx_defines NRF51xx Defines

@brief Defined Constants and Types for the NRF51xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

