/** @page libopencm3 PAC55xx

@version 1.0.0

API documentation for Qorvo PAC55xx series MCUs.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup PAC55xx_defines PAC55xx Defines

@brief Defined Constants and Types for the PAC55xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

