/** @page libopencm3 LM4F

@version 1.0.0

@date 22 November 2012

API documentation for TI Stellaris LM4F Cortex M4F series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LM4Fxx LM4F
Libraries for TI Stellaris  LM4F series.

@version 1.0.0

@date 22 November 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LM4Fxx_defines LM4F Defines

@brief Defined Constants and Types for the LM4F series

@version 1.0.0

@date 22 November 2012

LGPL License Terms @ref lgpl_license
*/

