/** @page libopencm3 NRF52

@version 1.0.0

@date November 2021

API documentation for Nordic Semiconductor nRF52 Cortex M4 series

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup NRF52xx NRF52xx
Libraries for Nordic Semiconductor NRF52xx series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup NRF52xx_defines NRF52xx Defines

@brief Defined Constants and Types for the NRF52xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

