/** @page libopencm3 STM32G0

@version 1.0.0

@date 30 January 2019

API documentation for ST Microelectronics STM32G0 Cortex M0+ series

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32G0xx STM32G0xx
Libraries for ST Microelectronics STM32G0xx series.

@version 1.0.0

@date 30 January 2019

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32G0xx_defines STM32G0xx Defines

@brief Defined Constants and Types for the STM32G0xx series

@version 1.0.0

@date 30 January 2019

LGPL License Terms @ref lgpl_license
*/

