/** @page libopencm3 STM32F1

@version 1.0.0

@date 7 September 2012

API documentation for ST Microelectronics STM32F1 Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32F1xx STM32F1xx
Libraries for ST Microelectronics STM32F1xx series.

@version 1.0.0

@date 7 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32F1xx_defines STM32F1xx Defines

@brief Defined Constants and Types for the STM32F1xx series

@version 1.0.0

@date 7 September 2012

LGPL License Terms @ref lgpl_license
*/

