/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define MEMCTL_IRQHandler memctl_isr
#define WDT_IRQHandler wdt_isr
#define RTC_IRQHandler rtc_isr
#define ADC0_IRQHandler adc0_isr
#define ADC1_IRQHandler adc1_isr
#define ADC2_IRQHandler adc2_isr
#define ADC3_IRQHandler adc3_isr
#define TIMERA_IRQHandler timera_isr
#define TIMERB_IRQHandler timerb_isr
#define TIMERC_IRQHandler timerc_isr
#define TIMERD_IRQHandler timerd_isr
#define TIMERA_QEP_IRQHandler timera_qep_isr
#define TIMERB_QEP_IRQHandler timerb_qep_isr
#define TIMERC_QEP_IRQHandler timerc_qep_isr
#define TIMERD_QEP_IRQHandler timerd_qep_isr
#define GPIOA_IRQHandler gpioa_isr
#define GPIOB_IRQHandler gpiob_isr
#define GPIOC_IRQHandler gpioc_isr
#define GPIOD_IRQHandler gpiod_isr
#define GPIOE_IRQHandler gpioe_isr
#define GPIOF_IRQHandler gpiof_isr
#define GPIOG_IRQHandler gpiog_isr
#define I2C_IRQHandler i2c_isr
#define USARTA_IRQHandler usarta_isr
#define USARTB_IRQHandler usartb_isr
#define USARTC_IRQHandler usartc_isr
#define USARTD_IRQHandler usartd_isr
#define CAN_IRQHandler can_isr
#define GPTIMERA_IRQHandler gptimera_isr
#define GPTIMERB_IRQHandler gptimerb_isr
#define SCC_IRQHandler scc_isr
