/** @page libopencm3 SWM050

@version 1.0.0

API documentation for Synwit SWM050 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup SWM050 SWM050
Libraries for Synwit SWM050 series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup SWM050_defines SWM050 Defines

@brief Defined Constants and Types for the SWM050 series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

