/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define RTC_IRQHandler rtc_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_1_IRQHandler exti0_1_isr
#define EXTI2_3_IRQHandler exti2_3_isr
#define EXTI4_15_IRQHandler exti4_15_isr
#define TSC_IRQHandler tsc_isr
#define DMA_CHANNEL1_IRQHandler dma_channel1_isr
#define DMA_CHANNEL2_3_IRQHandler dma_channel2_3_isr
#define DMA_CHANNEL4_5_IRQHandler dma_channel4_5_isr
#define ADC_COMP_IRQHandler adc_comp_isr
#define TIM1_BRK_UP_TRG_COM_IRQHandler tim1_brk_up_trg_com_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM6_DAC_IRQHandler tim6_dac_isr
#define RESERVED0_IRQHandler reserved0_isr
#define TIM14_IRQHandler tim14_isr
#define TIM15_IRQHandler tim15_isr
#define TIM16_IRQHandler tim16_isr
#define TIM17_IRQHandler tim17_isr
#define I2C1_EV_IRQHandler i2c1_ev_isr
#define I2C2_EV_IRQHandler i2c2_ev_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define RESERVED1_IRQHandler reserved1_isr
#define CEC_CAN_IRQHandler cec_can_isr
#define RESERVED2_IRQHandler reserved2_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define RESERVED3_IRQHandler reserved3_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define I2C3_EV_IRQHandler i2c3_ev_isr
#define I2C3_ER_IRQHandler i2c3_er_isr
#define USB_LP_IRQHandler usb_lp_isr
#define USB_HP_IRQHandler usb_hp_isr
#define RESERVED4_IRQHandler reserved4_isr
#define RESERVED5_IRQHandler reserved5_isr
#define RESERVED6_IRQHandler reserved6_isr
#define USB_WAKEUP_IRQHandler usb_wakeup_isr
#define RESERVED7_IRQHandler reserved7_isr
#define RESERVED8_IRQHandler reserved8_isr
#define RESERVED9_IRQHandler reserved9_isr
#define RESERVED10_IRQHandler reserved10_isr
#define RESERVED11_IRQHandler reserved11_isr
#define DMA_CHANNEL6_7_IRQHandler dma_channel6_7_isr
#define RESERVED12_IRQHandler reserved12_isr
#define RESERVED13_IRQHandler reserved13_isr
#define SPI3_IRQHandler spi3_isr
