/** @page libopencm3 STM32F3
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * API documentation for ST Microelectronics STM32F3 Cortex M3 series.
 *
 * LGPL License Terms @ref lgpl_license
 */

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32F3xx STM32F3xx
 * Libraries for ST Microelectronics STM32F3xx series.
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * LGPL License Terms @ref lgpl_license
 */

/** @defgroup STM32F3xx_defines STM32F3xx Defines
 *
 * @brief Defined Constants and Types for the STM32F3xx series
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * LGPL License Terms @ref lgpl_license
 */

