/** @page libopencm3 LPC43xx

@version 1.0.0

@date 14 September 2012

API documentation for NXP Semiconductors LPC43xx Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC43xx LPC43xx
Libraries for NXP Semiconductors LPC43xx series.

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC43xx_defines LPC43xx Defines

@brief Defined Constants and Types for the LPC43xx series

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

