/** @page libopencm3 EZR32 Wonder Gecko

@version 1.0.0

@date 14 January 2016

API documentation for Silicon Laboratories EZR32 Wonder Gecko Cortex M4 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EZR32WG EZR32 WonderGecko
Libraries for Silicon Laboratories EZR32 Wonder Gecko series.

@version 1.0.0

@date 14 January 2016

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup EZR32WG_defines EZR32 Wonder Gecko Defines

@brief Defined Constants and Types for the Silicon Laboratories EZR32
Wonder Gecko series

@version 1.0.0

@date 12 January 2016

LGPL License Terms @ref lgpl_license
*/

