/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define PM_IRQHandler pm_isr
#define SYSCTRL_IRQHandler sysctrl_isr
#define WDT_IRQHandler wdt_isr
#define RTC_IRQHandler rtc_isr
#define EIC_IRQHandler eic_isr
#define NVMCTRL_IRQHandler nvmctrl_isr
#define DMAC_IRQHandler dmac_isr
#define RESERVED1_IRQHandler reserved1_isr
#define EVSYS_IRQHandler evsys_isr
#define SERCOM0_IRQHandler sercom0_isr
#define SERCOM1_IRQHandler sercom1_isr
#define SERCOM2_IRQHandler sercom2_isr
#define TCC0_IRQHandler tcc0_isr
#define TC1_IRQHandler tc1_isr
#define TC2_IRQHandler tc2_isr
#define ADC_IRQHandler adc_isr
#define AC_IRQHandler ac_isr
#define DAC_IRQHandler dac_isr
#define PTC_IRQHandler ptc_isr
