/** @page libopencm3 VF6xx
 *
 * @version 1.0.0
 *
 * @date 03 July 2014
 *
 * API documentation for Freescale VF6xx series Cortex-M4 core.
 *
 * LGPL License Terms @ref lgpl_license
 */

/** @defgroup VF6xx VF6xx
 * Libraries for Freescale VF6xx series Cortex-M4 core.
 *
 * @version 1.0.0
 *
 * @date 03 July 2014
 *
 * LGPL License Terms @ref lgpl_license
 */

/** @defgroup VF6xx_defines VF6xx Defines
 *
 * @brief Defined Constants and Types for the VF6xx series
 *
 * @version 1.0.0
 *
 * @date 03 July 2014
 *
 * LGPL License Terms @ref lgpl_license
 */

