/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define HFLASHC_IRQHandler hflashc_isr
#define PDCA0_IRQHandler pdca0_isr
#define PDCA1_IRQHandler pdca1_isr
#define PDCA2_IRQHandler pdca2_isr
#define PDCA3_IRQHandler pdca3_isr
#define PDCA4_IRQHandler pdca4_isr
#define PDCA5_IRQHandler pdca5_isr
#define PDCA6_IRQHandler pdca6_isr
#define PDCA7_IRQHandler pdca7_isr
#define PDCA8_IRQHandler pdca8_isr
#define PDCA9_IRQHandler pdca9_isr
#define PDCA10_IRQHandler pdca10_isr
#define PDCA11_IRQHandler pdca11_isr
#define PDCA12_IRQHandler pdca12_isr
#define PDCA13_IRQHandler pdca13_isr
#define PDCA14_IRQHandler pdca14_isr
#define PDCA15_IRQHandler pdca15_isr
#define CRCCU_IRQHandler crccu_isr
#define USBC_IRQHandler usbc_isr
#define PEVC_TR_IRQHandler pevc_tr_isr
#define PEVC_OV_IRQHandler pevc_ov_isr
#define AESA_IRQHandler aesa_isr
#define PM_IRQHandler pm_isr
#define SCIM_IRQHandler scim_isr
#define FREQM_IRQHandler freqm_isr
#define GPIO0_IRQHandler gpio0_isr
#define GPIO1_IRQHandler gpio1_isr
#define GPIO2_IRQHandler gpio2_isr
#define GPIO3_IRQHandler gpio3_isr
#define GPIO4_IRQHandler gpio4_isr
#define GPIO5_IRQHandler gpio5_isr
#define GPIO6_IRQHandler gpio6_isr
#define GPIO7_IRQHandler gpio7_isr
#define GPIO8_IRQHandler gpio8_isr
#define GPIO9_IRQHandler gpio9_isr
#define GPIO10_IRQHandler gpio10_isr
#define GPIO11_IRQHandler gpio11_isr
#define BPM_IRQHandler bpm_isr
#define BSCIF_IRQHandler bscif_isr
#define AST_ALARM_IRQHandler ast_alarm_isr
#define AST_PER_IRQHandler ast_per_isr
#define AST_OVF_IRQHandler ast_ovf_isr
#define AST_READY_IRQHandler ast_ready_isr
#define AST_CLKREADY_IRQHandler ast_clkready_isr
#define WDT_IRQHandler wdt_isr
#define EIC1_IRQHandler eic1_isr
#define EIC2_IRQHandler eic2_isr
#define EIC3_IRQHandler eic3_isr
#define EIC4_IRQHandler eic4_isr
#define EIC5_IRQHandler eic5_isr
#define EIC6_IRQHandler eic6_isr
#define EIC7_IRQHandler eic7_isr
#define EIC8_IRQHandler eic8_isr
#define IISC_IRQHandler iisc_isr
#define SPI_IRQHandler spi_isr
#define TC00_IRQHandler tc00_isr
#define TC01_IRQHandler tc01_isr
#define TC02_IRQHandler tc02_isr
#define TC10_IRQHandler tc10_isr
#define TC11_IRQHandler tc11_isr
#define TC12_IRQHandler tc12_isr
#define TWIM0_IRQHandler twim0_isr
#define TWIS0_IRQHandler twis0_isr
#define TWIM1_IRQHandler twim1_isr
#define TWIS1_IRQHandler twis1_isr
#define USART0_IRQHandler usart0_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define ADCIFE_IRQHandler adcife_isr
#define DACC_IRQHandler dacc_isr
#define ACIFC_IRQHandler acifc_isr
#define ABDACB_IRQHandler abdacb_isr
#define TRNG_IRQHandler trng_isr
#define PARC_IRQHandler parc_isr
#define CATB_IRQHandler catb_isr
#define RESERVED0_IRQHandler reserved0_isr
#define TWIM2_IRQHandler twim2_isr
#define TWIM3_IRQHandler twim3_isr
#define LCDCA_IRQHandler lcdca_isr
