/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define PIO0_0_IRQHandler pio0_0_isr
#define PIO0_1_IRQHandler pio0_1_isr
#define PIO0_2_IRQHandler pio0_2_isr
#define PIO0_3_IRQHandler pio0_3_isr
#define PIO0_4_IRQHandler pio0_4_isr
#define PIO0_5_IRQHandler pio0_5_isr
#define PIO0_6_IRQHandler pio0_6_isr
#define PIO0_7_IRQHandler pio0_7_isr
#define PIO0_8_IRQHandler pio0_8_isr
#define PIO0_9_IRQHandler pio0_9_isr
#define PIO0_10_IRQHandler pio0_10_isr
#define PIO0_11_IRQHandler pio0_11_isr
#define PIO1_0_IRQHandler pio1_0_isr
#define PIO1_1_IRQHandler pio1_1_isr
#define PIO1_2_IRQHandler pio1_2_isr
#define PIO1_3_IRQHandler pio1_3_isr
#define PIO1_4_IRQHandler pio1_4_isr
#define PIO1_5_IRQHandler pio1_5_isr
#define PIO1_6_IRQHandler pio1_6_isr
#define PIO1_7_IRQHandler pio1_7_isr
#define PIO1_8_IRQHandler pio1_8_isr
#define PIO1_9_IRQHandler pio1_9_isr
#define PIO1_10_IRQHandler pio1_10_isr
#define PIO1_11_IRQHandler pio1_11_isr
#define PIO2_0_IRQHandler pio2_0_isr
#define PIO2_1_IRQHandler pio2_1_isr
#define PIO2_2_IRQHandler pio2_2_isr
#define PIO2_3_IRQHandler pio2_3_isr
#define PIO2_4_IRQHandler pio2_4_isr
#define PIO2_5_IRQHandler pio2_5_isr
#define PIO2_6_IRQHandler pio2_6_isr
#define PIO2_7_IRQHandler pio2_7_isr
#define PIO2_8_IRQHandler pio2_8_isr
#define PIO2_9_IRQHandler pio2_9_isr
#define PIO2_10_IRQHandler pio2_10_isr
#define PIO2_11_IRQHandler pio2_11_isr
#define PIO3_0_IRQHandler pio3_0_isr
#define PIO3_1_IRQHandler pio3_1_isr
#define PIO3_2_IRQHandler pio3_2_isr
#define PIO3_3_IRQHandler pio3_3_isr
#define I2C0_IRQHandler i2c0_isr
#define CT16B0_IRQHandler ct16b0_isr
#define CT16B1_IRQHandler ct16b1_isr
#define CT32B0_IRQHandler ct32b0_isr
#define CT32B1_IRQHandler ct32b1_isr
#define SSP0_IRQHandler ssp0_isr
#define UART_IRQHandler uart_isr
#define USB_IRQHandler usb_isr
#define USB_FIQ_IRQHandler usb_fiq_isr
#define ADC_IRQHandler adc_isr
#define WDT_IRQHandler wdt_isr
#define BOD_IRQHandler bod_isr
#define PIO3_IRQHandler pio3_isr
#define PIO2_IRQHandler pio2_isr
#define PIO1_IRQHandler pio1_isr
#define SSP1_IRQHandler ssp1_isr
