/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define DAC_IRQHandler dac_isr
#define M0CORE_IRQHandler m0core_isr
#define DMA_IRQHandler dma_isr
#define ETHERNET_IRQHandler ethernet_isr
#define SDIO_IRQHandler sdio_isr
#define LCD_IRQHandler lcd_isr
#define USB0_IRQHandler usb0_isr
#define USB1_IRQHandler usb1_isr
#define SCT_IRQHandler sct_isr
#define RITIMER_IRQHandler ritimer_isr
#define TIMER0_IRQHandler timer0_isr
#define TIMER1_IRQHandler timer1_isr
#define TIMER2_IRQHandler timer2_isr
#define TIMER3_IRQHandler timer3_isr
#define MCPWM_IRQHandler mcpwm_isr
#define ADC0_IRQHandler adc0_isr
#define I2C0_IRQHandler i2c0_isr
#define I2C1_IRQHandler i2c1_isr
#define SPI_IRQHandler spi_isr
#define ADC1_IRQHandler adc1_isr
#define SSP0_IRQHandler ssp0_isr
#define SSP1_IRQHandler ssp1_isr
#define USART0_IRQHandler usart0_isr
#define UART1_IRQHandler uart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define I2S0_IRQHandler i2s0_isr
#define I2S1_IRQHandler i2s1_isr
#define SPIFI_IRQHandler spifi_isr
#define SGPIO_IRQHandler sgpio_isr
#define PIN_INT0_IRQHandler pin_int0_isr
#define PIN_INT1_IRQHandler pin_int1_isr
#define PIN_INT2_IRQHandler pin_int2_isr
#define PIN_INT3_IRQHandler pin_int3_isr
#define PIN_INT4_IRQHandler pin_int4_isr
#define PIN_INT5_IRQHandler pin_int5_isr
#define PIN_INT6_IRQHandler pin_int6_isr
#define PIN_INT7_IRQHandler pin_int7_isr
#define GINT0_IRQHandler gint0_isr
#define GINT1_IRQHandler gint1_isr
#define EVENTROUTER_IRQHandler eventrouter_isr
#define C_CAN1_IRQHandler c_can1_isr
#define ATIMER_IRQHandler atimer_isr
#define RTC_IRQHandler rtc_isr
#define WWDT_IRQHandler wwdt_isr
#define C_CAN0_IRQHandler c_can0_isr
#define QEI_IRQHandler qei_isr
