/** @defgroup crs_defines CRS Defines
 *
 * @brief <b>Defined Constants and Types for the Clock Recovery System</b>
 *
 * @ingroup STM32G4xx_defines
 *
 *LGPL License Terms @ref lgpl_license
 */

#pragma once

#include <libopencm3/stm32/common/crs_common_all.h>
