/** @page libopencm3 EFM32 Leopard Gecko

@version 1.0.0

@date 4 March 2013

API documentation for Energy Micro EFM32 Leopard Gecko Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EFM32LG EFM32 LeopardGecko
Libraries for Energy Micro EFM32 Leopard Gecko series.

@version 1.0.0

@date 4 March 2013

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup EFM32LG_defines EFM32 Leopard Gecko Defines

@brief Defined Constants and Types for the Energy Micro EFM32 Leopard Gecko
series

@version 1.0.0

@date 4 March 2013

LGPL License Terms @ref lgpl_license
*/

