/** @page libopencm3 GD32F1x0

@version 1.0.0

API documentation for GigaDevices GD32F1x0 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup GD32F1x0 GD32F1x0xx
Libraries for GigaDevices GD32F1x0xx series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup GD32F1x0_defines GD32F1x0xx Defines

@brief Defined Constants and Types for the GD32F1x0xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

