/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define TAMP_STAMP_IRQHandler tamp_stamp_isr
#define RTC_WKUP_IRQHandler rtc_wkup_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_IRQHandler exti0_isr
#define EXTI1_IRQHandler exti1_isr
#define EXTI2_IRQHandler exti2_isr
#define EXTI3_IRQHandler exti3_isr
#define EXTI4_IRQHandler exti4_isr
#define DMA1_STR0_IRQHandler dma1_str0_isr
#define DMA1_STR1_IRQHandler dma1_str1_isr
#define DMA1_STR2_IRQHandler dma1_str2_isr
#define DMA1_STR3_IRQHandler dma1_str3_isr
#define DMA1_STR4_IRQHandler dma1_str4_isr
#define DMA1_STR5_IRQHandler dma1_str5_isr
#define DMA1_STR6_IRQHandler dma1_str6_isr
#define ADC1_2_IRQHandler adc1_2_isr
#define FDCAN1_IT0_IRQHandler fdcan1_it0_isr
#define FDCAN2_IT0_IRQHandler fdcan2_it0_isr
#define FDCAN1_IT1_IRQHandler fdcan1_it1_isr
#define FDCAN2_IT1_IRQHandler fdcan2_it1_isr
#define EXTI9_5_IRQHandler exti9_5_isr
#define TIM1_BRK_TIM9_IRQHandler tim1_brk_tim9_isr
#define TIM1_UP_TIM10_IRQHandler tim1_up_tim10_isr
#define TIM1_TRG_COM_TIM11_IRQHandler tim1_trg_com_tim11_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM4_IRQHandler tim4_isr
#define I2C1_EV_IRQHandler i2c1_ev_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define I2C2_EV_IRQHandler i2c2_ev_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define EXTI15_10_IRQHandler exti15_10_isr
#define RTC_ALARM_IRQHandler rtc_alarm_isr
#define RESERVED1_IRQHandler reserved1_isr
#define TIM8_BRK_TIM12_IRQHandler tim8_brk_tim12_isr
#define TIM8_UP_TIM13_IRQHandler tim8_up_tim13_isr
#define TIM8_TRG_COM_TIM14_IRQHandler tim8_trg_com_tim14_isr
#define TIM8_CC_IRQHandler tim8_cc_isr
#define DMA1_STR7_IRQHandler dma1_str7_isr
#define FSMC_IRQHandler fsmc_isr
#define SDMMC1_IRQHandler sdmmc1_isr
#define TIM5_IRQHandler tim5_isr
#define SPI3_IRQHandler spi3_isr
#define UART4_IRQHandler uart4_isr
#define UART5_IRQHandler uart5_isr
#define TIM6_DAC_IRQHandler tim6_dac_isr
#define TIM7_IRQHandler tim7_isr
#define DMA2_STR0_IRQHandler dma2_str0_isr
#define DMA2_STR1_IRQHandler dma2_str1_isr
#define DMA2_STR2_IRQHandler dma2_str2_isr
#define DMA2_STR3_IRQHandler dma2_str3_isr
#define DMA2_STR4_IRQHandler dma2_str4_isr
#define ETH_IRQHandler eth_isr
#define ETH_WKUP_IRQHandler eth_wkup_isr
#define FDCAN_CAL_IRQHandler fdcan_cal_isr
#define CM7_SEV_IRQHandler cm7_sev_isr
#define RESERVED2_IRQHandler reserved2_isr
#define RESERVED3_IRQHandler reserved3_isr
#define RESERVED4_IRQHandler reserved4_isr
#define DMA2_STR5_IRQHandler dma2_str5_isr
#define DMA2_STR6_IRQHandler dma2_str6_isr
#define DMA2_STR7_IRQHandler dma2_str7_isr
#define USART6_IRQHandler usart6_isr
#define I2C3_EV_IRQHandler i2c3_ev_isr
#define I2C3_ER_IRQHandler i2c3_er_isr
#define OTG_HS_EP1_OUT_IRQHandler otg_hs_ep1_out_isr
#define OTG_HS_EP1_IN_IRQHandler otg_hs_ep1_in_isr
#define OTG_HS_WKUP_IRQHandler otg_hs_wkup_isr
#define OTG_HS_IRQHandler otg_hs_isr
#define DCMI_IRQHandler dcmi_isr
#define CRYP_IRQHandler cryp_isr
#define HASH_RNG_IRQHandler hash_rng_isr
#define FPU_IRQHandler fpu_isr
#define UART7_IRQHandler uart7_isr
#define UART8_IRQHandler uart8_isr
#define SPI4_IRQHandler spi4_isr
#define SPI5_IRQHandler spi5_isr
#define SPI6_IRQHandler spi6_isr
#define SAI1_IRQHandler sai1_isr
#define LTDC_IRQHandler ltdc_isr
#define LTDC_ER_IRQHandler ltdc_er_isr
#define DMA2D_IRQHandler dma2d_isr
#define SAI2_IRQHandler sai2_isr
#define QUADSPI_IRQHandler quadspi_isr
#define LP_TIM1_IRQHandler lp_tim1_isr
#define CEC_IRQHandler cec_isr
#define I2C4_EV_IRQHandler i2c4_ev_isr
#define I2C4_ER_IRQHandler i2c4_er_isr
#define SPDIFRX_IRQHandler spdifrx_isr
#define OTG_FS_EP1_OUT_IRQHandler otg_fs_ep1_out_isr
#define OTG_FS_EP1_IN_IRQHandler otg_fs_ep1_in_isr
#define OTG_FS_WKUP_IRQHandler otg_fs_wkup_isr
#define OTG_FS_IRQHandler otg_fs_isr
#define DMAMUX1_OV_IRQHandler dmamux1_ov_isr
#define HRTIM1_MST_IRQHandler hrtim1_mst_isr
#define HRTIM1_TIMA_IRQHandler hrtim1_tima_isr
#define HRTIM1_TIMB_IRQHandler hrtim1_timb_isr
#define HRTIM1_TIMC_IRQHandler hrtim1_timc_isr
#define HRTIM1_TIMD_IRQHandler hrtim1_timd_isr
#define HRTIM1_TIME_IRQHandler hrtim1_time_isr
#define HRTIM1_FLT_IRQHandler hrtim1_flt_isr
#define DFSDM1_IT0_IRQHandler dfsdm1_it0_isr
#define DFSDM1_IT1_IRQHandler dfsdm1_it1_isr
#define DFSDM1_IT2_IRQHandler dfsdm1_it2_isr
#define DFSDM1_IT3_IRQHandler dfsdm1_it3_isr
#define SAI3_IRQHandler sai3_isr
#define SWPMI1_IRQHandler swpmi1_isr
#define TIM15_IRQHandler tim15_isr
#define TIM16_IRQHandler tim16_isr
#define TIM17_IRQHandler tim17_isr
#define MDIOS_WKUP_IRQHandler mdios_wkup_isr
#define MDIOS_IRQHandler mdios_isr
#define JPEG_IRQHandler jpeg_isr
#define MDMA_IRQHandler mdma_isr
#define RESERVED5_IRQHandler reserved5_isr
#define SDMMC2_IRQHandler sdmmc2_isr
#define HSEM0_IRQHandler hsem0_isr
#define RESERVED6_IRQHandler reserved6_isr
#define ADC3_IRQHandler adc3_isr
#define DMAMUX2_OVR_IRQHandler dmamux2_ovr_isr
#define BDMA_CH0_IRQHandler bdma_ch0_isr
#define BDMA_CH1_IRQHandler bdma_ch1_isr
#define BDMA_CH2_IRQHandler bdma_ch2_isr
#define BDMA_CH3_IRQHandler bdma_ch3_isr
#define BDMA_CH4_IRQHandler bdma_ch4_isr
#define BDMA_CH5_IRQHandler bdma_ch5_isr
#define BDMA_CH6_IRQHandler bdma_ch6_isr
#define BDMA_CH7_IRQHandler bdma_ch7_isr
#define COMP_IRQHandler comp_isr
#define LPTIM2_IRQHandler lptim2_isr
#define LPTIM3_IRQHandler lptim3_isr
#define LPTIM4_IRQHandler lptim4_isr
#define LPTIM5_IRQHandler lptim5_isr
#define LPUART_IRQHandler lpuart_isr
#define WWDG1_RST_IRQHandler wwdg1_rst_isr
#define CRS_IRQHandler crs_isr
#define RAMECC1_IRQHandler ramecc1_isr
#define SAI4_IRQHandler sai4_isr
#define RESERVED7_IRQHandler reserved7_isr
#define RESERVED8_IRQHandler reserved8_isr
#define WKUP_IRQHandler wkup_isr
