/** @page libopencm3 STM32G4

@version 1.0.0

@date April 2020

API documentation for ST Microelectronics STM32G4 Cortex M4 series

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32G4xx STM32G4xx
Libraries for ST Microelectronics STM32G4xx series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32G4xx_defines STM32G4xx Defines

@brief Defined Constants and Types for the STM32G4xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

