/** @page libopencm3 STM32F2

@version 1.0.0

@date 14 September 2012

API documentation for ST Microelectronics STM32F2 Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/


/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32F2xx STM32F2xx
Libraries for ST Microelectronics STM32F2xx series.

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32F2xx_defines STM32F2xx Defines

@brief Defined Constants and Types for the STM32F2xx series

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

