/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define SUPC_IRQHandler supc_isr
#define RSTC_IRQHandler rstc_isr
#define RTC_IRQHandler rtc_isr
#define RTT_IRQHandler rtt_isr
#define WDT_IRQHandler wdt_isr
#define PMC_IRQHandler pmc_isr
#define EEFC0_IRQHandler eefc0_isr
#define EEFC1_IRQHandler eefc1_isr
#define UART_IRQHandler uart_isr
#define SMC_SDRAMC_IRQHandler smc_sdramc_isr
#define SDRAMC_IRQHandler sdramc_isr
#define PIOA_IRQHandler pioa_isr
#define PIOB_IRQHandler piob_isr
#define PIOC_IRQHandler pioc_isr
#define PIOD_IRQHandler piod_isr
#define PIOE_IRQHandler pioe_isr
#define PIOF_IRQHandler piof_isr
#define USART0_IRQHandler usart0_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define HSMCI_IRQHandler hsmci_isr
#define TWI0_IRQHandler twi0_isr
#define TWI1_IRQHandler twi1_isr
#define SPI0_IRQHandler spi0_isr
#define SPI1_IRQHandler spi1_isr
#define SSC_IRQHandler ssc_isr
#define TC0_IRQHandler tc0_isr
#define TC1_IRQHandler tc1_isr
#define TC2_IRQHandler tc2_isr
#define TC3_IRQHandler tc3_isr
#define TC4_IRQHandler tc4_isr
#define TC5_IRQHandler tc5_isr
#define TC6_IRQHandler tc6_isr
#define TC7_IRQHandler tc7_isr
#define TC8_IRQHandler tc8_isr
#define PWM_IRQHandler pwm_isr
#define ADC_IRQHandler adc_isr
#define DACC_IRQHandler dacc_isr
#define DMAC_IRQHandler dmac_isr
#define UOTGHS_IRQHandler uotghs_isr
#define TRNG_IRQHandler trng_isr
#define RESERVED0_IRQHandler reserved0_isr
#define CAN0_IRQHandler can0_isr
#define CAN1_IRQHandler can1_isr
