/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define RTC_IRQHandler rtc_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_1_IRQHandler exti0_1_isr
#define EXTI2_3_IRQHandler exti2_3_isr
#define EXTI4_15_IRQHandler exti4_15_isr
#define UCPD1_UCPD2_IRQHandler ucpd1_ucpd2_isr
#define DMA1_CHANNEL1_IRQHandler dma1_channel1_isr
#define DMA1_CHANNEL2_3_IRQHandler dma1_channel2_3_isr
#define DMA1_CHANNEL4_7_DMAMUX_IRQHandler dma1_channel4_7_dmamux_isr
#define ADC_COMP_IRQHandler adc_comp_isr
#define TIM1_BRK_UP_TRG_COM_IRQHandler tim1_brk_up_trg_com_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM34_IRQHandler tim34_isr
#define TIM6_DAC_LPTIM1_IRQHandler tim6_dac_lptim1_isr
#define TIM7_LPTIM2_IRQHandler tim7_lptim2_isr
#define TIM14_IRQHandler tim14_isr
#define TIM15_IRQHandler tim15_isr
#define TIM16_FDCAN_IT0_IRQHandler tim16_fdcan_it0_isr
#define TIM17_FDCAN_IT1_IRQHandler tim17_fdcan_it1_isr
#define I2C1_IRQHandler i2c1_isr
#define I2C23_IRQHandler i2c23_isr
#define SPI1_IRQHandler spi1_isr
#define SPI23_IRQHandler spi23_isr
#define USART1_IRQHandler usart1_isr
#define USART2_LPUART2_IRQHandler usart2_lpuart2_isr
#define USART3456_LPUART1_IRQHandler usart3456_lpuart1_isr
#define CEC_IRQHandler cec_isr
#define AES_RNG_IRQHandler aes_rng_isr
