/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WDT_IRQHandler wdt_isr
#define TIMER0_IRQHandler timer0_isr
#define TIMER1_IRQHandler timer1_isr
#define TIMER2_IRQHandler timer2_isr
#define TIMER3_IRQHandler timer3_isr
#define UART0_IRQHandler uart0_isr
#define UART1_IRQHandler uart1_isr
#define UART2_IRQHandler uart2_isr
#define UART3_IRQHandler uart3_isr
#define PWM_IRQHandler pwm_isr
#define I2C0_IRQHandler i2c0_isr
#define I2C1_IRQHandler i2c1_isr
#define I2C2_IRQHandler i2c2_isr
#define SPI_IRQHandler spi_isr
#define SSP0_IRQHandler ssp0_isr
#define SSP1_IRQHandler ssp1_isr
#define PLL0_IRQHandler pll0_isr
#define RTC_IRQHandler rtc_isr
#define EINT0_IRQHandler eint0_isr
#define EINT1_IRQHandler eint1_isr
#define EINT2_IRQHandler eint2_isr
#define EINT3_IRQHandler eint3_isr
#define ADC_IRQHandler adc_isr
#define BOD_IRQHandler bod_isr
#define USB_IRQHandler usb_isr
#define CAN_IRQHandler can_isr
#define GPDMA_IRQHandler gpdma_isr
#define I2S_IRQHandler i2s_isr
#define ETHERNET_IRQHandler ethernet_isr
#define RIT_IRQHandler rit_isr
#define MOTOR_PWM_IRQHandler motor_pwm_isr
#define QEI_IRQHandler qei_isr
#define PLL1_IRQHandler pll1_isr
#define USB_ACT_IRQHandler usb_act_isr
#define CAN_ACT_IRQHandler can_act_isr
