/** @page libopencm3 STM32L0

@version 1.0.0

@date 15 November 2014

API documentation for ST Microelectronics STM32L0 Cortex M0 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32L0xx STM32L0xx
Libraries for ST Microelectronics STM32L0xx series.

@version 1.0.0

@date 15 November 2014

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32L0xx_defines STM32L0xx Defines

@brief Defined Constants and Types for the STM32L0xx series

@version 1.0.0

@date 7 September 2012

LGPL License Terms @ref lgpl_license
*/

