/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define TIMER_SE0_IRQHandler timer_se0_isr
#define TIMER_SE1_IRQHandler timer_se1_isr
#define WDT_IRQHandler wdt_isr
#define CP_IRQHandler cp_isr
#define GPIOA0_IRQHandler gpioa0_isr
#define GPIOA1_IRQHandler gpioa1_isr
#define GPIOA2_IRQHandler gpioa2_isr
#define GPIOA3_IRQHandler gpioa3_isr
#define GPIOA4_IRQHandler gpioa4_isr
#define GPIOA5_IRQHandler gpioa5_isr
#define GPIOA6_IRQHandler gpioa6_isr
#define GPIOA7_IRQHandler gpioa7_isr
#define GPIOA8_IRQHandler gpioa8_isr
#define GPIOA9_IRQHandler gpioa9_isr
