/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define DMA_IRQHandler dma_isr
#define GPIO_EVEN_IRQHandler gpio_even_isr
#define TIMER0_IRQHandler timer0_isr
#define USART0_RX_IRQHandler usart0_rx_isr
#define USART0_TX_IRQHandler usart0_tx_isr
#define ACMP01_IRQHandler acmp01_isr
#define ADC0_IRQHandler adc0_isr
#define DAC0_IRQHandler dac0_isr
#define I2C0_IRQHandler i2c0_isr
#define GPIO_ODD_IRQHandler gpio_odd_isr
#define TIMER1_IRQHandler timer1_isr
#define USART1_RX_IRQHandler usart1_rx_isr
#define USART1_TX_IRQHandler usart1_tx_isr
#define LESENSE_IRQHandler lesense_isr
#define LEUART0_IRQHandler leuart0_isr
#define LETIMER0_IRQHandler letimer0_isr
#define PCNT0_IRQHandler pcnt0_isr
#define RTC_IRQHandler rtc_isr
#define CMU_IRQHandler cmu_isr
#define VCMP_IRQHandler vcmp_isr
#define LCD_IRQHandler lcd_isr
#define MSC_IRQHandler msc_isr
#define AES_IRQHandler aes_isr
