/** @page libopencm3 STM32H7

@version 1.0.0

API documentation for ST Microelectronics STM32H7 Cortex M7 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32H7xx STM32H7xx
Libraries for ST Microelectronics STM32H7xx series.

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32H7xx_defines STM32Hxx Defines

@brief Defined Constants and Types for the STM32H7xx series

@version 1.0.0

LGPL License Terms @ref lgpl_license
*/

