/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define TAMPER_STAMP_IRQHandler tamper_stamp_isr
#define RTC_WKUP_IRQHandler rtc_wkup_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_IRQHandler exti0_isr
#define EXTI1_IRQHandler exti1_isr
#define EXTI2_IRQHandler exti2_isr
#define EXTI3_IRQHandler exti3_isr
#define EXTI4_IRQHandler exti4_isr
#define DMA1_CHANNEL1_IRQHandler dma1_channel1_isr
#define DMA1_CHANNEL2_IRQHandler dma1_channel2_isr
#define DMA1_CHANNEL3_IRQHandler dma1_channel3_isr
#define DMA1_CHANNEL4_IRQHandler dma1_channel4_isr
#define DMA1_CHANNEL5_IRQHandler dma1_channel5_isr
#define DMA1_CHANNEL6_IRQHandler dma1_channel6_isr
#define DMA1_CHANNEL7_IRQHandler dma1_channel7_isr
#define ADC1_IRQHandler adc1_isr
#define USB_HP_IRQHandler usb_hp_isr
#define USB_LP_IRQHandler usb_lp_isr
#define DAC_IRQHandler dac_isr
#define COMP_IRQHandler comp_isr
#define EXTI9_5_IRQHandler exti9_5_isr
#define LCD_IRQHandler lcd_isr
#define TIM9_IRQHandler tim9_isr
#define TIM10_IRQHandler tim10_isr
#define TIM11_IRQHandler tim11_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM4_IRQHandler tim4_isr
#define I2C1_EV_IRQHandler i2c1_ev_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define I2C2_EV_IRQHandler i2c2_ev_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define EXTI15_10_IRQHandler exti15_10_isr
#define RTC_ALARM_IRQHandler rtc_alarm_isr
#define USB_FS_WAKEUP_IRQHandler usb_fs_wakeup_isr
#define TIM6_IRQHandler tim6_isr
#define TIM7_IRQHandler tim7_isr
#define SDIO_IRQHandler sdio_isr
#define TIM5_IRQHandler tim5_isr
#define SPI3_IRQHandler spi3_isr
#define UART4_IRQHandler uart4_isr
#define UART5_IRQHandler uart5_isr
#define DMA2_CH1_IRQHandler dma2_ch1_isr
#define DMA2_CH2_IRQHandler dma2_ch2_isr
#define DMA2_CH3_IRQHandler dma2_ch3_isr
#define DMA2_CH4_IRQHandler dma2_ch4_isr
#define DMA2_CH5_IRQHandler dma2_ch5_isr
#define AES_IRQHandler aes_isr
#define COMP_ACQ_IRQHandler comp_acq_isr
