/** @page libopencm3 LM3S

@version 1.0.0

@date 14 September 2012

API documentation for TI Stellaris LM3S Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LM3Sxx LM3S
Libraries for TI Stellaris  LM3S series.

@version 1.0.0

@date 7 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LM3Sxx_defines LM3S Defines

@brief Defined Constants and Types for the LM3S series

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

