/** @page libopencm3 STM32L4

@version 1.0.0

@date 12 February 2015

API documentation for ST Microelectronics STM32L4 Cortex M4 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32L4xx STM32L4xx
Libraries for ST Microelectronics STM32L4xx series.

@version 1.0.0

@date 12 February 2015

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32L4xx_defines STM32L4xx Defines

@brief Defined Constants and Types for the STM32L4xx series

@version 1.0.0

@date 12 February 2015

LGPL License Terms @ref lgpl_license
*/

