/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define POWER_CLOCK_IRQHandler power_clock_isr
#define RADIO_IRQHandler radio_isr
#define UART0_IRQHandler uart0_isr
#define SPI0_TWI0_IRQHandler spi0_twi0_isr
#define SPI1_TWI1_IRQHandler spi1_twi1_isr
#define RESERVED0_IRQHandler reserved0_isr
#define GPIOTE_IRQHandler gpiote_isr
#define ADC_IRQHandler adc_isr
#define TIMER0_IRQHandler timer0_isr
#define TIMER1_IRQHandler timer1_isr
#define TIMER2_IRQHandler timer2_isr
#define TIMER3_IRQHandler timer3_isr
#define TIMER4_IRQHandler timer4_isr
#define RTC0_IRQHandler rtc0_isr
#define TEMP_IRQHandler temp_isr
#define RNG_IRQHandler rng_isr
#define ECB_IRQHandler ecb_isr
#define CCM_AAR_IRQHandler ccm_aar_isr
#define WDT_IRQHandler wdt_isr
#define RTC1_IRQHandler rtc1_isr
#define QDEC_IRQHandler qdec_isr
#define LPCOMP_IRQHandler lpcomp_isr
#define SWI0_IRQHandler swi0_isr
#define SWI1_IRQHandler swi1_isr
#define SWI2_IRQHandler swi2_isr
#define SWI3_IRQHandler swi3_isr
#define SWI4_IRQHandler swi4_isr
#define SWI5_IRQHandler swi5_isr
