/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define RTC_TAMP_CSS_IRQHandler rtc_tamp_css_isr
#define RTC_WKUP_IRQHandler rtc_wkup_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_IRQHandler exti0_isr
#define EXTI1_IRQHandler exti1_isr
#define EXTI2_IRQHandler exti2_isr
#define EXTI3_IRQHandler exti3_isr
#define EXTI4_IRQHandler exti4_isr
#define DMA1_CHANNEL1_IRQHandler dma1_channel1_isr
#define DMA1_CHANNEL2_IRQHandler dma1_channel2_isr
#define DMA1_CHANNEL3_IRQHandler dma1_channel3_isr
#define DMA1_CHANNEL4_IRQHandler dma1_channel4_isr
#define DMA1_CHANNEL5_IRQHandler dma1_channel5_isr
#define DMA1_CHANNEL6_IRQHandler dma1_channel6_isr
#define DMA1_CHANNEL7_IRQHandler dma1_channel7_isr
#define ADC12_IRQHandler adc12_isr
#define USB_HP_IRQHandler usb_hp_isr
#define USB_LP_IRQHandler usb_lp_isr
#define FDCAN1_IT0_IRQHandler fdcan1_it0_isr
#define FDCAN1_IT1_IRQHandler fdcan1_it1_isr
#define EXTI9_5_IRQHandler exti9_5_isr
#define TIM1_BRK_TIM15_IRQHandler tim1_brk_tim15_isr
#define TIM1_UP_TIM16_IRQHandler tim1_up_tim16_isr
#define TIM1_TRG_TIM17_IRQHandler tim1_trg_tim17_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM4_IRQHandler tim4_isr
#define I2C1_EV_IRQHandler i2c1_ev_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define I2C2_EV_IRQHandler i2c2_ev_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define EXTI15_10_IRQHandler exti15_10_isr
#define RTC_ALARM_IRQHandler rtc_alarm_isr
#define USB_WAKEUP_IRQHandler usb_wakeup_isr
#define TIM8_BRK_IRQHandler tim8_brk_isr
#define TIM8_UP_IRQHandler tim8_up_isr
#define TIM8_TRG_IRQHandler tim8_trg_isr
#define TIM8_CC_IRQHandler tim8_cc_isr
#define ADC3_IRQHandler adc3_isr
#define FSMC_IRQHandler fsmc_isr
#define LPTIM1_IRQHandler lptim1_isr
#define TIM5_IRQHandler tim5_isr
#define SPI3_IRQHandler spi3_isr
#define UART4_IRQHandler uart4_isr
#define UART5_IRQHandler uart5_isr
#define TIM6_DAC13UNDER_IRQHandler tim6_dac13under_isr
#define TIM7_DAC24UNDER_IRQHandler tim7_dac24under_isr
#define DMA2_CHANNEL1_IRQHandler dma2_channel1_isr
#define DMA2_CHANNEL2_IRQHandler dma2_channel2_isr
#define DMA2_CHANNEL3_IRQHandler dma2_channel3_isr
#define DMA2_CHANNEL4_IRQHandler dma2_channel4_isr
#define DMA2_CHANNEL5_IRQHandler dma2_channel5_isr
#define ADC4_IRQHandler adc4_isr
#define ADC5_IRQHandler adc5_isr
#define UCPD1_IRQHandler ucpd1_isr
#define COMP123_IRQHandler comp123_isr
#define COMP456_IRQHandler comp456_isr
#define COMP7_IRQHandler comp7_isr
#define HRTIM_MASTER_IRQHandler hrtim_master_isr
#define HRTIM_TIMA_IRQHandler hrtim_tima_isr
#define HRTIM_TIMB_IRQHandler hrtim_timb_isr
#define HRTIM_TIMC_IRQHandler hrtim_timc_isr
#define HRTIM_TIMD_IRQHandler hrtim_timd_isr
#define HRTIM_TIME_IRQHandler hrtim_time_isr
#define HRTIM_FAULT_IRQHandler hrtim_fault_isr
#define HRTIM_TIMF_IRQHandler hrtim_timf_isr
#define CRS_IRQHandler crs_isr
#define SAI_IRQHandler sai_isr
#define TIM20_BRK_IRQHandler tim20_brk_isr
#define TIM20_UP_IRQHandler tim20_up_isr
#define TIM20_TRG_IRQHandler tim20_trg_isr
#define TIM20_CC_IRQHandler tim20_cc_isr
#define FPU_IRQHandler fpu_isr
#define I2C4_EV_IRQHandler i2c4_ev_isr
#define I2C4_ER_IRQHandler i2c4_er_isr
#define SPI4_IRQHandler spi4_isr
#define AES_IRQHandler aes_isr
#define FDCAN2_IT0_IRQHandler fdcan2_it0_isr
#define FDCAN2_IT1_IRQHandler fdcan2_it1_isr
#define FDCAN3_IT0_IRQHandler fdcan3_it0_isr
#define FDCAN3_IT1_IRQHandler fdcan3_it1_isr
#define RNG_IRQHandler rng_isr
#define LPUART_IRQHandler lpuart_isr
#define I2C3_EV_IRQHandler i2c3_ev_isr
#define I2C3_ER_IRQHandler i2c3_er_isr
#define DMAMUX_OVR_IRQHandler dmamux_ovr_isr
#define QUADSPI_IRQHandler quadspi_isr
#define DMA1_CHANNEL8_IRQHandler dma1_channel8_isr
#define DMA2_CHANNEL6_IRQHandler dma2_channel6_isr
#define DMA2_CHANNEL7_IRQHandler dma2_channel7_isr
#define DMA2_CHANNEL8_IRQHandler dma2_channel8_isr
#define CORDIC_IRQHandler cordic_isr
#define FMAC_IRQHandler fmac_isr
