/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define RTC_IRQHandler rtc_isr
#define M4CORE_IRQHandler m4core_isr
#define DMA_IRQHandler dma_isr
#define FLASHEEPROMAT_IRQHandler flasheepromat_isr
#define ETHERNET_IRQHandler ethernet_isr
#define SDIO_IRQHandler sdio_isr
#define LCD_IRQHandler lcd_isr
#define USB0_IRQHandler usb0_isr
#define USB1_IRQHandler usb1_isr
#define SCT_IRQHandler sct_isr
#define RITIMER_OR_WWDT_IRQHandler ritimer_or_wwdt_isr
#define TIMER0_IRQHandler timer0_isr
#define GINT1_IRQHandler gint1_isr
#define PIN_INT4_IRQHandler pin_int4_isr
#define TIMER3_IRQHandler timer3_isr
#define MCPWM_IRQHandler mcpwm_isr
#define ADC0_IRQHandler adc0_isr
#define I2C0_OR_IRC1_IRQHandler i2c0_or_irc1_isr
#define SGPIO_IRQHandler sgpio_isr
#define SPI_OR_DAC_IRQHandler spi_or_dac_isr
#define ADC1_IRQHandler adc1_isr
#define SSP0_OR_SSP1_IRQHandler ssp0_or_ssp1_isr
#define EVENTROUTER_IRQHandler eventrouter_isr
#define USART0_IRQHandler usart0_isr
#define UART1_IRQHandler uart1_isr
#define USART2_OR_C_CAN1_IRQHandler usart2_or_c_can1_isr
#define USART3_IRQHandler usart3_isr
#define I2S0_OR_I2S1_IRQHandler i2s0_or_i2s1_isr
#define C_CAN0_IRQHandler c_can0_isr
