/** @page libopencm3 EFM32 Happy Gecko

@version 1.0.0

@date 28 January 2018

API documentation for Energy Micro EFM32 Happy Gecko Cortex M0+ series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EFM32HG EFM32 HappyGecko
Libraries for Energy Micro EFM32 Happy Gecko series.

@version 1.0.0

@date 28 January 2018

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup EFM32HG_defines EFM32 Happy Gecko Defines

@brief Defined Constants and Types for the Energy Micro EFM32 Happy Gecko
series

@version 1.0.0

@date 28 January 2018

LGPL License Terms @ref lgpl_license
*/
