/** @mainpage libopencm3 Core CM3

@version 1.0.0

@date 14 September 2012

API documentation for Cortex M3 core features.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup CM3_defines CM3 Defines

@brief Defined Constants and Types for Cortex M3 core features

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup CM3_files Cortex Core Peripheral APIs
 * APIs for Cortex Core peripherals
 */

