/** @page libopencm3 EFM32 Gecko

@version 1.0.0

@date 11 November 2012

API documentation for Energy Micro EFM32 Gecko Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EFM32G EFM32 Gecko
Libraries for Energy Micro EFM32 Gecko series.

@version 1.0.0

@date 11 November 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup EFM32G_defines EFM32 Gecko Defines

@brief Defined Constants and Types for the Energy Micro EFM32 Gecko series

@version 1.0.0

@date 11 November 2012

LGPL License Terms @ref lgpl_license
*/

