/** @page libopencm3 LPC13xx

@version 1.0.0

@date 14 September 2012

API documentation for NXP Semiconductors LPC13xx Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC13xx LPC13xx
Libraries for NXP Semiconductors LPC13xx series.

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup LPC13xx_defines LPC13xx Defines

@brief Defined Constants and Types for the LPC13xx series

@version 1.0.0

@date 14 September 2012

LGPL License Terms @ref lgpl_license
*/

