/** @page libopencm3 MSP432E4

@version 1.0.0

@date 29 July 2018

API documentation for Texas Instruments MSP432E4xx Cortex M4F series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup MSP432E4xx MSP432E4xx
Libraries for Texas Instruments MSP432E4xx series.

@version 1.0.0

@date 29 July 2018

LGPL License Terms @ref lgpl_license
*/

/** @defgroup MSP432E4xx_defines MSP432E4xx Defines
 *
@brief Defined Constants and Types for the MSP432E4xx series.

@version 1.0.0

@date 29 July 2018

LGPL License Terms @ref lgpl_license
*/
