/** @page libopencm3 STM32F7

@version 1.0.0

@date 15 September 2014

API documentation for ST Microelectronics STM32F7 Cortex M7 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup peripheral_apis Peripheral APIs
 * APIs for device peripherals
 */

/** @defgroup STM32F7xx STM32F7xx
Libraries for ST Microelectronics STM32F7xx series.

@version 1.0.0

@date 15 September 2014

LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32F7xx_defines STM32F7xx Defines

@brief Defined Constants and Types for the STM32F7xx series

@version 1.0.0

@date 15 September 2014

LGPL License Terms @ref lgpl_license
*/

