/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define NVIC_WWDG_IRQHandler nvic_wwdg_isr
#define PVD_IRQHandler pvd_isr
#define TAMP_STAMP_IRQHandler tamp_stamp_isr
#define RTC_WKUP_IRQHandler rtc_wkup_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_IRQHandler exti0_isr
#define EXTI1_IRQHandler exti1_isr
#define EXTI2_TSC_IRQHandler exti2_tsc_isr
#define EXTI3_IRQHandler exti3_isr
#define EXTI4_IRQHandler exti4_isr
#define DMA1_CHANNEL1_IRQHandler dma1_channel1_isr
#define DMA1_CHANNEL2_IRQHandler dma1_channel2_isr
#define DMA1_CHANNEL3_IRQHandler dma1_channel3_isr
#define DMA1_CHANNEL4_IRQHandler dma1_channel4_isr
#define DMA1_CHANNEL5_IRQHandler dma1_channel5_isr
#define DMA1_CHANNEL6_IRQHandler dma1_channel6_isr
#define DMA1_CHANNEL7_IRQHandler dma1_channel7_isr
#define ADC1_2_IRQHandler adc1_2_isr
#define USB_HP_CAN1_TX_IRQHandler usb_hp_can1_tx_isr
#define USB_LP_CAN1_RX0_IRQHandler usb_lp_can1_rx0_isr
#define CAN1_RX1_IRQHandler can1_rx1_isr
#define CAN1_SCE_IRQHandler can1_sce_isr
#define EXTI9_5_IRQHandler exti9_5_isr
#define TIM1_BRK_TIM15_IRQHandler tim1_brk_tim15_isr
#define TIM1_UP_TIM16_IRQHandler tim1_up_tim16_isr
#define TIM1_TRG_COM_TIM17_IRQHandler tim1_trg_com_tim17_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM4_IRQHandler tim4_isr
#define I2C1_EV_EXTI23_IRQHandler i2c1_ev_exti23_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define I2C2_EV_EXTI24_IRQHandler i2c2_ev_exti24_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_EXTI25_IRQHandler usart1_exti25_isr
#define USART2_EXTI26_IRQHandler usart2_exti26_isr
#define USART3_EXTI28_IRQHandler usart3_exti28_isr
#define EXTI15_10_IRQHandler exti15_10_isr
#define RTC_ALARM_IRQHandler rtc_alarm_isr
#define USB_WKUP_A_IRQHandler usb_wkup_a_isr
#define TIM8_BRK_IRQHandler tim8_brk_isr
#define TIM8_UP_IRQHandler tim8_up_isr
#define TIM8_TRG_COM_IRQHandler tim8_trg_com_isr
#define TIM8_CC_IRQHandler tim8_cc_isr
#define ADC3_IRQHandler adc3_isr
#define RESERVED_1_IRQHandler reserved_1_isr
#define RESERVED_2_IRQHandler reserved_2_isr
#define RESERVED_3_IRQHandler reserved_3_isr
#define SPI3_IRQHandler spi3_isr
#define UART4_EXTI34_IRQHandler uart4_exti34_isr
#define UART5_EXTI35_IRQHandler uart5_exti35_isr
#define TIM6_DAC_IRQHandler tim6_dac_isr
#define TIM7_IRQHandler tim7_isr
#define DMA2_CHANNEL1_IRQHandler dma2_channel1_isr
#define DMA2_CHANNEL2_IRQHandler dma2_channel2_isr
#define DMA2_CHANNEL3_IRQHandler dma2_channel3_isr
#define DMA2_CHANNEL4_IRQHandler dma2_channel4_isr
#define DMA2_CHANNEL5_IRQHandler dma2_channel5_isr
#define ETH_IRQHandler eth_isr
#define RESERVED_4_IRQHandler reserved_4_isr
#define RESERVED_5_IRQHandler reserved_5_isr
#define COMP123_IRQHandler comp123_isr
#define COMP456_IRQHandler comp456_isr
#define COMP7_IRQHandler comp7_isr
#define HRTIM_MASTER_IRQHandler hrtim_master_isr
#define HRTIM_TIMA_IRQHandler hrtim_tima_isr
#define HRTIM_TIMB_IRQHandler hrtim_timb_isr
#define HRTIM_TIMC_IRQHandler hrtim_timc_isr
#define HRTIM_TIMD_IRQHandler hrtim_timd_isr
#define HRTIM_TIME_IRQHandler hrtim_time_isr
#define HRTIM_FLT_IRQHandler hrtim_flt_isr
#define USB_HP_IRQHandler usb_hp_isr
#define USB_LP_IRQHandler usb_lp_isr
#define USB_WKUP_IRQHandler usb_wkup_isr
#define RESERVED_13_IRQHandler reserved_13_isr
#define RESERVED_14_IRQHandler reserved_14_isr
#define RESERVED_15_IRQHandler reserved_15_isr
#define RESERVED_16_IRQHandler reserved_16_isr
